/** A test Phenotype for the simple paraboloide problem f(x, y) = x^2 + y^2 
* using two Chromosomes (x, y). The task is to find the minimum.
*
* @version 0.4
* @author Helmut A. Mayer
* @since June 2, 2000
*
*/

import java.util.*;

import evSOLve.JEvolution.*;


public class ParaboloidePhenotype implements Phenotype {


	private double x;						// paraboloide phenotype x
	private double y;						// paraboloide phenotype y
	private double fitness;					
				
		
	public ParaboloidePhenotype() {
		x = 0.0;							// Java default, not really necessary
		fitness = 0.0;						// also
	}
		

/** A proper clone. Here the phenotype is simply a double, so super.clone() handles
* everything just fine.
*
* @returns Object	the cloned phenotype
*
*/

	public Object clone() {
		try {
			ParaboloidePhenotype clone = (ParaboloidePhenotype)super.clone();
			return(clone);
		} catch (CloneNotSupportedException e) {
			throw new InternalError(e.toString());						// should not happen
		}
	}
	

/** The genotype-phenotype mapper converts the two BitChromosomes to 
* the phenotype values x, y.
*/
	
	public void doOntogeny(Vector genotype) {
	
		int actualValue, maxValue;

		for (int i = 0; i < 2; i++) {									// two Chromosomes
			BitChromosome bitChrom = (BitChromosome)genotype.get(i);	// get single Chromosome
			BitSet bits = (BitSet)bitChrom.getBases();					// access bases
			int chromosomeLength = bitChrom.getLength();
		
			actualValue = maxValue = 0;
			for (int j = 0; j < chromosomeLength; j++) {				// bits to real number
				actualValue <<= 1;
				maxValue <<= 1;
				if (bits.get(j))
					actualValue |= 1;
				maxValue |= 1;
			}
			if (i == 0)
				x = (double)actualValue / (double)maxValue;
			else
				y = (double)actualValue / (double)maxValue;
		}
	}	
			
				
/** The fitness function x^2 + y^2. */
	
		public void calcFitness() {
		
			fitness = Math.pow(x, 2.0) + Math.pow(y, 2.0);
			fitness = 1.0 / (1.0 + fitness);							// minimization problem
		}
		
	
/** Access to the fitness of the Phenotype. */
	
	public double getFitness() {
		return fitness;
	}
	

/** The String representation of the Phenotype values x, y. */

	public String toString() {
		return("x: " + x + " y: " + y);
	}
	
}